#include <Servo.h>

// Define pinout
#define LED PIN_PA3
#define ESC_PIN PIN_PA4
#define SERVO_SIG1 PIN_PA7
#define SERVO_SIG2 PIN_PA1
#define SERVO_SIG3 PIN_PB0
#define SERVO_SIG4 PIN_PB1
#define SBUS_RX PIN_PA2
#define RX PIN_PB3
#define TX PIN_PB2

// Create servo objects
Servo servo1;
Servo servo2;
Servo servo3;
Servo servo4;
Servo servoESC;
int esc_speed = 0; // value from the analog pin
int servo_pos = 0;

// Array storing all servos
Servo servo_arr[] = {servo1, servo1, servo2, servo3, servo4}; // to support indexing from 1, add a dummy copy to the beginning

// LED controls
const int interval = 1000; // interval at which to blink in milliseconds
unsigned long previousMillis = 0;  // will store last time LED was updated
int ledState = LOW;

void blink();
void help_menu();

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);

  // Attach servos
  servo1.attach(SERVO_SIG1);
  servo2.attach(SERVO_SIG2);
  servo3.attach(SERVO_SIG3);
  servo4.attach(SERVO_SIG4);

  // Arm the ESC
  servoESC.attach(ESC_PIN, 1000, 2000); // (pin, min pulse width, max pulse width)
  servoESC.write(esc_speed);

  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);

  delay(3000);
  // loops are for luzers
  servo1.writeMicroseconds(1500); // assume 1500 is the neutral position 
  servo2.writeMicroseconds(1500); 
  servo3.writeMicroseconds(1500);  
  servo4.writeMicroseconds(1500);
}

void loop() {
  // put your main code here, to run repeatedly:
  blink();

  if (Serial.available() > 0) {
  
    // Serial.println("Enter request ");
    String msg = Serial.readString();
    // Serial.println(msg);

    char ctrl_bit = msg[0];

    if (ctrl_bit == '0') { // pwm signal for ESC BLDC
      digitalWrite(LED, ledState);
      int pwm = msg.substring(1).toInt();
      servoESC.write(pwm); // send the signal to the ESC

    } else if (ctrl_bit == '1' || ctrl_bit == '2' || ctrl_bit == '3' || ctrl_bit == '4') { // servo control 

      // why do this the smart way??
      int servo_index;
      if (ctrl_bit == '1')
        servo_index = 1;
      if (ctrl_bit == '2')
        servo_index = 2;
      if (ctrl_bit == '3')
        servo_index = 3;
      if (ctrl_bit == '4')
        servo_index = 4;

      char direction_bit = msg[1];
      int microseconds;
      if (direction_bit == '0') { // go left
        microseconds = 1000;
        ledState = HIGH;
      } else if (direction_bit == '1') { // go right
        microseconds = 2000;
        ledState = HIGH;
      } else { // incorrect 
        Serial.println("Direction bit for servo control must be either 0 or 1");
        microseconds = 1500;
      }
      digitalWrite(LED, ledState);
      servo_arr[servo_index].writeMicroseconds(microseconds);
    } else { // not supported
      Serial.print("Control bit ");
      Serial.print(ctrl_bit); 
      Serial.println(" not supported");
      help_menu();
    }
  }
    
  //   // Read the value and write to controller
  //   esc_speed = Serial.parseInt();
  //   if (esc_speed < 1000 || esc_speed > 2000) {
  //     Serial.println("Value must be between 1000 and 2000");
  //   } else {
  //     Serial.print("Wrote ");
  //     Serial.print(esc_speed);
  //     Serial.println(" to ESC");
  //     servoESC.write(esc_speed); // send the signal to the ESC
  //   }
  // }

  // Spin servos 180 degrees
  // for (servo_pos = 0; servo_pos <= 180; servo_pos++) {
  //   servo1.write(servo_pos);
  //   servo2.write(servo_pos);
  //   servo3.write(servo_pos);
  //   servo4.write(servo_pos);
  //   delay(15); // wait 15ms for the servos to reach position
  // }

}

void blink() {
  // Check to see if it's time to blink the LED; that is, if the difference
  // between the current time and last time you blinked the LED is bigger than
  // the interval at which you want to blink the LED.
  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {
    // save the last time you blinked the LED
    previousMillis = currentMillis;

    // if the LED is off turn it on and vice-versa:
    if (ledState == LOW) {
      ledState = HIGH;
    } else {
      ledState = LOW;
    }

    // set the LED with the ledState of the variable:
    digitalWrite(LED, ledState);
  }
}

void help_menu() {
  Serial.print("Usage: [ctrl_bit][signal]\nIf [ctrl_bit] == 0 then pwm signal expected for BLDC\nIf [ctrl_bit] in [1...4] then [direction_bit] expect that is either 0 (left) or 1 (right)\n");
}
